var initSummernote = function() {
    $('#summernote').summernote({
        buttons: {
            editLink: buttonEditLink
        },
        minHeight: MRichEditor.getMinHeight(),
        maxHeight: null,
        focus: false,
        disableResizeEditor: true,
        toolbar: [],
        popover: {
            link: [
                ['link', ['editLink', 'unlink']]
            ]
        },
        placeholder: MRichEditor.getPlaceholder(),
        callbacks: {
            onInit: function(e) {
                $('.note-statusbar').hide();
                if (MRichEditor.fullscreen()) {
                    $("#summernote").summernote("fullscreen.toggle");
                }

                $("#summernote").on('summernote.keyup summernote.mouseup summernote.change', function () {
                    var style = $("#summernote").summernote("editor.currentStyle");
                    var rng = $('#summernote').summernote('createRange');
                    var hasQuote = style.ancestors.find(isBlockQuote) != null;
                    var hasLink = rng.isCollapsed() && rng.isOnAnchor();
                    var hasTextSize = style.ancestors.find(isH4) != null;
                    var hasUnorderedList = style.ancestors.find(isUnorderedList) != null;

                    MRichEditor.updateCurrentStyle(hasTextSize && !hasQuote, hasUnorderedList, hasQuote, hasLink);
                });

                initialTextSize = $("#summernote").summernote("editor.currentStyle")["font-size"];
                MRichEditor.onPageLoaded();
            },

            onChange: function(contents, $editable) {
                MRichEditor.onChange(contents);
            }
        }
    });
}

var lastLinkInfo = null;
var buttonEditLink = function (context) {
  var ui = $.summernote.ui;
  var options = $.summernote.options;
  var lang = $.summernote.lang["en-US"];

  // create button
  var button = ui.button({
    contents: ui.icon(options.icons.link),
    click: function () {

      lastLinkInfo = context.invoke('editor.getLinkInfo');
      context.invoke('editor.saveRange');
      $('.popover').hide();

      MRichEditor.onLinkEdit(lastLinkInfo.text, lastLinkInfo.url, true);
    }
  });

  return button.render();
}

var setWindowHeight = function(height) {
    document.getElementsByClassName('note-editable')[0].style.height = height;
}

function isUnorderedList(ancestor) {
    return ancestor.localName == "ul";
}

var disableUnorderedList = function() {
    var style = $("#summernote").summernote("editor.currentStyle");
    var ul = style.ancestors.find(isUnorderedList);
    if (ul != null) {
        toggleUnorderedList();
    }
};

var toggleUnorderedList = function() {
    $('#summernote').summernote('insertUnorderedList');
};

function isBlockQuote(ancestor) {
    return ancestor.localName == "blockquote";
}

var toggleBlockquote = function() {
    var style = $("#summernote").summernote("editor.currentStyle");
    var quote = style.ancestors.find(isBlockQuote);
    if (quote != null) {
        formatPara();
    } else {
        formatBlock('blockquote');
    }
}

var disableBlockquote = function() {
    var style = $("#summernote").summernote("editor.currentStyle");
    var quote = style.ancestors.find(isBlockQuote);
    if (quote != null) {
        formatPara();
    }
}

function isH4(ancestor) {
    return ancestor.localName == "h4";
}

var toggleTextSize = function() {
    var style = $("#summernote").summernote("editor.currentStyle");
    var h4 = style.ancestors.find(isH4);
    if (h4 == null) {
        formatH4();
    } else {
        formatPara();
    }
}

var disableTextSize = function() {
    var style = $("#summernote").summernote("editor.currentStyle");
    var h4 = style.ancestors.find(isH4);
    if (h4 != null) {
        formatPara();
    }
}

var obtainCurrentSelection = function() {
    MRichEditor.onSelectionReady($('#summernote').summernote('createRange').toString());
}

var prepareUrlDialog = function() {
    var selection = $('#summernote').summernote('createRange').toString();
    MRichEditor.onLinkEdit(selection, null, false);
}

var pasteHTML = function(html) {
    $("#summernote").summernote('code', html);

    if (html.length > 0) {
        $('.note-placeholder').hide();
    }

    if (!MRichEditor.fullscreen()) {
        setTimeout(function() {
            focus();
            scrollToBottom();
            keepLastIndex();
        }, 50);
    }
}

function keepLastIndex() {
    var obj = document.getElementsByClassName('note-editable')[0];
    var range = window.getSelection();
    range.selectAllChildren(obj);
    range.collapseToEnd();
}

var scrollToBottom = function() {
    $("html, body").animate({ scrollTop: $(document).height() }, 450);
}

var focus = function() {
    $('#summernote').summernote('focus');
}

var undo = function() {
    $('#summernote').summernote('undo');
};

var redo = function() {
    $('#summernote').summernote('redo');
};

var disable = function() {
    $('#summernote').summernote('disable');
};

var enable = function() {
    $('#summernote').summernote('enable');
};

var removeFormat = function() {
    $('#summernote').summernote('removeFormat');
}

var bold = function() {
    $('#summernote').summernote('bold');
};

var italic = function() {
    $('#summernote').summernote('italic');
};

var underline = function() {
    $('#summernote').summernote('underline');
};

var strikethrough = function() {
    $('#summernote').summernote('strikethrough');
};

var superscript = function() {
    $('#summernote').summernote('superscript');
};

var subscript = function() {
    $('#summernote').summernote('subscript');
};

var backColor = function(color) {
    $('#summernote').summernote('backColor', color);
};

var foreColor = function(color) {
    $('#summernote').summernote('foreColor', color);
};

var fontName = function(fontName) {
    $('#summernote').summernote('fontName', fontName);
};

var fontSize = function(fontSize) {
    $('#summernote').summernote('fontSize', fontSize);
};

var justifyLeft = function() {
    $('#summernote').summernote('justifyLeft');
};

var justifyRight = function() {
    $('#summernote').summernote('justifyRight');
};

var justifyCenter = function() {
    $('#summernote').summernote('justifyCenter');
};

var justifyFull = function() {
    $('#summernote').summernote('justifyFull');
};

var insertOrderedList = function() {
    $('#summernote').summernote('insertOrderedList');
};

var insertUnorderedList = function() {
    $('#summernote').summernote('insertUnorderedList');
};

var indent = function() {
    $('#summernote').summernote('indent');
};

var outdent = function() {
    $('#summernote').summernote('outdent');
};

var formatBlock = function(tagName) {
    $('#summernote').summernote('formatBlock', tagName);
};

var formatPara = function() {
    $('#summernote').summernote('formatPara');
};

var formatH1 = function() {
    $('#summernote').summernote('formatH1');
};

var formatH2 = function() {
    $('#summernote').summernote('formatH2');
};

var formatH3 = function() {
    $('#summernote').summernote('formatH3');
};

var formatH4 = function() {
    $('#summernote').summernote('formatH4');
};

var formatH5 = function() {
    $('#summernote').summernote('formatH5');
};

var formatH6 = function() {
    $('#summernote').summernote('formatH6');
};

var lineHeight = function(lineHeight) {
    $('#summernote').summernote('lineHeight', lineHeight);
};

var insertImageUrl = function(imageUrl) {
    $('#summernote').summernote('insertImage', imageUrl, null);
};

var insertText = function(text) {
    $('#summernote').summernote('insertText', text);

    if (html.length > 0) {
        $('.note-placeholder').hide();
    }
};

var createLink = function(linkText, linkUrl) {
    $('#summernote').summernote('createLink', {
      text: linkText,
      url: linkUrl,
      isNewWindow: true
    });
};

var editLink = function(linkText, linkUrl) {
    if (lastLinkInfo == null) {
        return;
    }

    $('#summernote').summernote('editor.restoreRange');
    $('#summernote').summernote('createLink', {
      range: lastLinkInfo.range,
      text: linkText,
      url: linkUrl,
      isNewWindow: true
    });
}

var cancelLinkEdit = function() {
    $('#summernote').summernote('editor.restoreRange');
}

var unlink = function() {
    $('#summernote').summernote('unlink');
};

var codeView = function(){
    $('#summernote').summernote('codeview.toggle');
}

var insertTable = function(dim){
    $('#summernote').summernote('insertTable', dim);
}

var insertHorizontalRule = function() {
    $('#summernote').summernote('insertHorizontalRule');
}
